#!/usr/bin/perl

use strict;
use Digest::MD5;

open(R,">revised_test_suite") or die "error opening revised_test_suite for output, $!";
open(B,">browser_test_output.xhtml") or die "error opening browser_test_output.xhtml for output, $!";

print B <<HEADER;
<?xml version="1.0" encoding="utf-8" ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN" "http://www.w3.org/Math/DTD/mathml2/xhtml-math11-f.dtd" >
<html xmlns="http://www.w3.org/1999/xhtml">
<head><title>foo</title><meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8"/></head><body>
HEADER

open(F,"<test_suite") or die "error opening test_suite for input, $!";
my @passed = ();
my @failed = ();
my @new_expected = ();
while (my $line = <F>) {
  if ($line eq 'end') {print R "\n"}
  print R $line;
  chomp $line;
  last if $line eq 'end';
  next if $line=~/\A\s*\Z/ || $line=~/\A\s*#/; # ignore blank lines and comment lines
  $line =~ /\A([^,]*),([^,]*),([^,]*),([^,]*),(.*)/ or die "illegal input line $line";
  my ($fmt,$prepend,$err,$options,$input) = ($1,$2,$3,$4,$5);
  my $comment = '';
  if ($input =~ /\A(.*)#(.*)\Z/) {$input = $1; $comment = $2}
  my @expect = ();
  for (;;) {
    my $expect_line = <F>;
    chomp $expect_line;
    last if $expect_line=~/\A\s*\Z/;
    $expect_line=~/\A  / or die "error in this line in test_suite:\n$expect_line\nDoes not begin with at least two blanks. Typically this happens because you don't have the required blank line between tests.";
    push @expect,$expect_line;
  }
  my $expect = join("\n",@expect);
  $expect='' if $expect=~/\A\s*\Z/;
  my $png = '';
  if ($fmt eq 'png') {$png = '--png-file test.png'}
  my $prepend_option = '';
  if ($prepend ne '') {
    open(PREPEND,">test.prepend") or die $!;
    print PREPEND $prepend;
    close PREPEND;
    $prepend_option = '--prepend-file test.prepend'
  }
  open(G,">test.tex") or die "error opening test.tex for output, $!";
  print G "$input\n";
  close G;
  my $c = "footex --$fmt $png $prepend_option test.tex test.html test_info 1>test.stdout 2>test.stderr";
  print (sprintf "%-50s %s","$input","# $comment\n");
  my $gory_details = 
         "$input\n  fmt=$fmt, prepend=$prepend, err=$err, options=$options, input=$input, comment=$comment\n  expect=$expect\n  $c\n";
  # ... save for later; two places where this might get printed
  # print $gory_details;
  my $got_err = (system($c)!=0);
  if (!$err) {
    my $stderr_contents = read_whole_file("test.stderr");
    print $stderr_contents if defined $stderr_contents;
  }

  # Three things that can happen:
  my $fail = ''; # null string or explanation of error
  my $pass; # boolean
  my $new_expected; # boolean

  my $image = ($fmt eq 'png');

  my $result = '';
  my $error_output = '';
  my $input_browserproofed = $input;
  $input_browserproofed =~ s/&/&amp;/g;
  $input_browserproofed =~ s/</&lt;/g;
  print B <<HTML;
            <p>##################### $comment</p>
            <pre>$input_browserproofed</pre>
HTML
  if ($got_err && $err) {$pass=1; print R format_expected_result($expect)}
  if ($got_err && !$err) {$fail="error where none was expected"; $pass=0; print R format_expected_result($expect); $error_output = read_whole_file("test.stdout").read_whole_file("test.stderr")}
  if (!$got_err && $err) {$fail="no error where one was expected"; $pass=0; print R format_expected_result($expect)}
  if (!$got_err && !$err) {
    if ($image) {
      my $info = read_whole_file("test_info"); # height and depth in pixels
      $info = join(",",split(/\n/,$info));
      $result = "  ".Digest::MD5::md5_hex(read_whole_file("test.png")).",".$info;
      my $code = Digest::MD5::md5_hex($line);
      my $image_file = "test_$code.png";
      system("mv test.png $image_file");
      print B <<HTML;
        <p><img src="$image_file"/></p>
HTML
    }
    else { # not image
      my $html = read_whole_file("test.html");
      die "no error from footex, but can't open test.html" if ! defined $html;
      # add indentation, remove blank lines
      my @html = split(/\n/,$html);
      my $indented = '';
      foreach my $l(@html) {$indented = $indented . "  $l\n" unless $l=~/\A\s*\Z/}
      $result = $indented;
      $result =~ s/\n\Z//;
      #print $result,"\n";
      if ($fmt eq 'mathml') {
        print B <<HTML;
            <p><math xmlns="http://www.w3.org/1998/Math/MathML">$result</math></p>
HTML
      }
      else {
        print B "<p>$result</p>\n"
      }
    } # end if not image
    if ($expect eq '') {$new_expected = 1; print R format_expected_result($result)}
    if ($expect ne '' && $expect ne $result) {print R format_expected_result($expect); $fail="output not as expected"}
    if ($expect ne '' && $expect eq $result) {print R format_expected_result($expect),"\n"; $pass=1}
  }
  if ($fail ne '') {
    print "  ********************* FAIL *******************\n  result=$result=\n  $fail\n";
    print B "<p>********************* FAIL *******************</p><p> $fail</p>\n";
    push @failed,$line;
    if ($error_output ne '') {print $error_output,"\n"}
    print $gory_details;
    print "  If you want to inspect these files, edit the source code so it stops here, preserving the files.\n";
  }
  if ($pass) {
    push @passed,$line;
  }
  if ($new_expected) {
    push @new_expected,$line;
  }
}
close F;
close R;

print "\n";

if (@failed>0) {
  print "Failed tests:\n".join("\n",@failed)."\n";
}
if (@new_expected>0) {
  print "Newly inferred correct outputs ( mv revised_test_suite test_suite ):\n".join("\n",@new_expected)."\n";
}
print "Passed ".@passed.", failed ".@failed.", new ".@new_expected."\n";
print "Browser-viewable results are in browser_test_output.xhtml\n";

print B <<FOOTER;
      </body></html>
FOOTER

sub read_whole_file {
  my $file = shift;
  open(QQQ,"<$file") or return undef;
  local $/; # slurp whole file
  my $x = <QQQ>;
  close QQQ;
  return $x;
}

sub format_expected_result {
  my $x = shift;
  $x = $x . "\n";
  $x =~ s/\n{2,}/\n/g;
  return $x;
}
