use strict;

my $html = <<'HTML_TEMPLATE';
 <HTML>
	<HEAD>
		<TITLE>
			FooTeX
		</TITLE>
		<LINK rel="stylesheet" href="../lightandmatter.css" type="text/css">
		<META NAME="description" content="FooTeX">
		<META NAME="keywords" content="calendar,software">
	</HEAD>
	<a name="top"></a>
	<BODY bgcolor="white">
	<img src="../banner.jpg" width="580" height="97"><br><!-- area1sales doesn't use this -->

	
	<table border=0 width="580" cellspacing="0" cellpadding="0">
	<!-- ==============Menu of main areas=============== -->
	<!-- ==============below the banner etc.=============== -->
    <tr><td>

		<!-- goes inside the table row -->
	

	<h1>FooTeX</h2>
<p>
FooTex is software for converting LaTeX math markup into mathml, html,
or png. It is basically just a wrapper for texvc and blahtex. It
provides some extra functionality, and gives a consistent interface
to the functionality of the two underlying packages.
The documentation is built into the source code. When you install
footex, the documentation will be installed as a man page.
</p>
<p>
Software: <a href="footex.tar.gz">footex.tar.gz</a>
</p>
<p>
For convenience, FooTex is distributed along with texvc and blahtex.
Texvc is part of MediaWiki. Blahtex was originally written by David Harvey,
and texvc by Tomasz Wegrzanowski. David Harvey no longer maintains
blahtex, but Gilles Van Assche packages an extended version of it
called blahtexml, which also includes the blahtex executable used
by FooTex; this is the version that is distributed along with footex.
I have removed some optimization options from blahtex's makefile,
because they were making my system spin in an endless loop. On
a BSD system, you'll need to use gmake rather than make.
</p>
<p>

The following is a list of debian packages that suffices to build
everything:
<pre>
  ocaml-core
  ocaml-doc
  dvipng
  texlive-full
  libxerces27-dev
</pre>

</p>
<p>
To build everything and install everything on linux:
<pre>
  make
  make install
</pre>
</p>
<p>

On BSD, you'll need to use gmake, and you may want to fiddle with
the directory the executables are installed into.
</p>
<h2>man page</h2>
<pre>
MANPAGE
</pre>
	
		<!-- ==============Footer=============== -->

	<table border=0 width="580" cellspacing="0" cellpadding="0" bgcolor="#eeeeee">
	<tr><td colspan="4">
	<hr>
	<center>[<a href="../index.html">Home</a>
				| <a href="../index.html#outline">Site Map</a>
				| <a href="../area4privacy.html">Privacy</a>
				| <a href="../area4author.html">Contact</a> ]</center>
	<p>
	<font face="Times New Roman,Times" color="gray" size="2">
	(c) Copyright 2003 Benjamin Crowell. All rights reserved.</font>
	</td>
	</tr>


	</table>
	</align>
	</BODY>
 </HTML>

HTML_TEMPLATE

my $man = `nroff -man footex.1`;
$man =~ s/(.)\x{08}./$1/g;
$man =~ s/[^ \w,.\-:;'"()[]{}<>+=|\\`~!?\/@#$%^&*_]//g;

my $version = `perl footex --bare_version`;
$version =~ m/((\d+\.)+\d+)/;
$version = $1;
# ... same logic as in Makefile

$html =~ s/VERSION/$version/g;
$html =~ s/MANPAGE/$man/g;

print clean_up_text($html);

#------------------

sub clean_up_text {
  # gets rid of Mac and Windows curly quotes
  # changes crlf to \n, etc.
  # converts utf8/Mac/Win m-dashes to --, and n-dashes to -
  my $text = shift;
  # curly quotes:
  $text =~ s/\xe2\x80(\x9c|\x9d)/"/g; # utf8 curly quotes
  $text =~ s/\xe2\x80(\x98|\x99)/'/g; # utf8 curly apostrophe/single quotes
  $text =~ s/\xef\xa3\xa7/--/g; # utf8 m-dash
  $text =~ s/\xe2\x80\x94/--/g; # another type of utf8 m-dash??
  $text =~ s/(\223|\224)/"/g; # octal; Windows curly quotes
  $text =~ s/(\221|\222)/'/g; # octal; Windows curly apostrophe/single quotes
  $text =~ s/\226/-/g; # Windows n-dash
  $text =~ s/\227/--/g; # Windows m-dash
  $text =~ s/(\xd2|\xd3)/"/g; # Mac curly quotes
  $text =~ s/(\xd4|\xd5)/"/g; # Mac curly apostrophe/single quotes
  $text =~ s/\xd0/-/g; # Mac n-dash
  $text =~ s/\xd1/--/g; # Mac m-dash
  $text =~ s/\xc9/.../g; # Mac ellipsis
  $text =~ s/\r\n/\n/g; # crlf -> newline
  $text =~ s/\r/\n/g; # cr -> newline
  $text =~ s/\342\200\220/-/g; # octal; some goofy version of a hyphen
  return $text;
}
